#include <limits.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <SDL.h>

#include "aim.h"


#if CHAR_BIT != 8
#error char size larger than 8 is not supported
#endif


static struct image *
image_init(int cols, int rows, pixval maxval)
{
	struct image *img = malloc(sizeof(*img));
	img->cols = cols, img->rows = rows; img->maxval = maxval;
	img->bitmap = malloc(img->rows * image_rowbytes(img) * sizeof(*img->bitmap));
	return img;
}

typedef int_fast64_t timestamp_t;

static timestamp_t
get_timer(void)
{
	struct timeval t;
	gettimeofday(&t, NULL);
	return 1000000*t.tv_sec + t.tv_usec;
}

static void
render(struct image *img)
{
	SDL_LockSurface(img->screen);
	exercise(NULL, img);
	SDL_UnlockSurface(img->screen);
	SDL_Flip(img->screen);
}

int
main(int argc, char *argv[])
{
	SDL_Surface *screen;

	if (argc != 4) {
		fprintf(stderr, "%s ROWS COLS BPP\n", argv[0]);
		return EXIT_FAILURE;
	}

	int bpp = atoi(argv[3]);
	struct image img = { .rows = atoi(argv[1]), .cols = atoi(argv[2]), .maxval = 1<<bpp };

	if (SDL_Init(SDL_INIT_VIDEO) < 0) return 1;
	if (!(screen = SDL_SetVideoMode(img.cols, img.rows, bpp, SDL_HWSURFACE))) {
		SDL_Quit();
		return 1;
	}
	img.screen = screen;
	img.bitmap = screen->pixels;

	render(&img);

	bool quit = false;
	while (!quit) {
		SDL_Event event;
		while(SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_QUIT:
				quit = 1; break;
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
					case SDLK_SPACE: render(&img); /* Redraw */ break;
					case SDLK_ESCAPE: quit = 1; break;
					default: break;
				}
				break;
			}
		}
	}

	SDL_Quit();
	return EXIT_SUCCESS;
}
